require 'spec_helper'

describe MdsFileUtils::ZipAnalyzer do

  context "when testing for a valid zip file" do

    context "and provided with a file that does not have a zip extension" do

      before(:each) do
        @file_to_test = File.join('/tmp', 'file_to_test.txt')
        File.open(@file_to_test, 'w') { |f| f.puts "TestFile" }
        @za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)
      end

      after(:each) do
        FileUtils.rm_f @file_to_test
      end

      it "should respond with false if asked if a file that has a non-zip extension is a valid zip" do
        @za.valid_zip?.should be_falsey
      end

    end

    context "and provided with an invalid zip file" do

      after(:each) do
        FileUtils.rm @file_to_test if @file_to_test.present?
      end

      it "should respond with false if asked if a text file is a valid zip" do
        @file_to_test = File.join('/tmp', 'file_to_test.txt')
        File.open(@file_to_test, 'w') { |f| f.puts "TestFile" }
        
        za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)

        za.valid_zip?.should be_falsey
      end

      it "should respond with false if asked if an image file is a valid zip" do
        @file_to_test = File.join('/tmp', 'file_to_test.gif')
        FileUtils.cp 'spec/test_data/clear.gif', @file_to_test

        za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)

        za.valid_zip?.should be_falsey
      end

    end

    context "and provided with a valid zip file" do

      after(:each) do
        FileUtils.rm @file_to_test if @file_to_test.present?
      end

      it "should respond with true if asked if a valid zip file has an extension of '.zip'" do
        @file_to_test = File.join('/tmp', 'file_to_test.zip')
        contained_file = File.join('/tmp', 'contained_file')
        File.open(contained_file, 'w') { |f| f.puts "Hello" }
        system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' '#{File.basename(contained_file)}' > /dev/null 2>&1"
        FileUtils.rm contained_file
        
        za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)

        za.valid_zip?.should be_truthy
      end

      it "should respond with true if asked if a valid zip file has an extension of '.ZIP'" do
        @file_to_test = File.join('/tmp', 'file_to_test.ZIP')
        contained_file = File.join('/tmp', 'contained_file')
        File.open(contained_file, 'w') { |f| f.puts "Hello" }
        system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' '#{File.basename(contained_file)}' > /dev/null 2>&1"
        FileUtils.rm contained_file
        
        za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)

        za.valid_zip?.should be_truthy
      end

    end

  end

  context "when testing whether the zip file is a composite MDS file" do

    after(:each) do
      FileUtils.rm @file_to_test if @file_to_test.present? && File.exists?(@file_to_test)
    end

    it "should respond with true when asked if the file composite for a zip that contains nothing but files with a zip extension" do
      @file_to_test = File.join('/tmp', 'file_to_test.zip')
      # Create the files to go into the zip file
      contained_paths = (1..2).map { |idx| File.join('/tmp', "contained_#{idx}.zip") }
      contained_paths.each { |path| File.open(path, 'w') { |f| f.puts "Hello #{path}" } }
      paths_to_add = contained_paths.map { |path| File.basename(path) }.join(' ')
      system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' #{paths_to_add}  > /dev/null 2>&1"
      contained_paths.each { |path| FileUtils.rm path }

      za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)
      
      za.is_composite?.should be_truthy
    end

    it "should respond with false when asked if the file is a composite and the zip contains only .xml files" do
      @file_to_test = File.join('/tmp', 'file_to_test.zip')
      # Create the files to go into the zip file
      contained_paths = (1..2).map { |idx| File.join('/tmp', "contained_#{idx}.xml") }
      contained_paths.each { |path| File.open(path, 'w') { |f| f.puts "Hello #{path}" } }
      paths_to_add = contained_paths.map { |path| File.basename(path) }.join(' ')
      system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' '#{paths_to_add}' > /dev/null 2>&1"
      contained_paths.each { |path| FileUtils.rm path }

      za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)
      
      za.is_composite?.should be_falsey
    end

    it "should respond with false when asked if the file is a composite and the zip contains multiple .zip files and a single .xml file" do
      @file_to_test = File.join('/tmp', 'file_to_test.zip')
      # Create the files to go into the zip file
      contained_paths = (1..4).map { |idx| extension = idx < 4 ? 'zip' : 'xml'; File.join('/tmp', "contained_#{idx}.#{extension}") }
      contained_paths.each { |path| File.open(path, 'w') { |f| f.puts "Hello #{path}" } }
      paths_to_add = contained_paths.map { |path| File.basename(path) }.join(' ')
      system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' '#{paths_to_add}' > /dev/null 2>&1"
      contained_paths.each { |path| FileUtils.rm path }

      za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)
      
      za.is_composite?.should be_falsey
    end

    it "should respond with false when asked if the file is a composite and the zip contains files with various extensions but no .zip files" do
      EXTENSIONS ||= %W{ xml gif txt }
      @file_to_test = File.join('/tmp', 'file_to_test.zip')
      # Create the files to go into the zip file
      contained_paths = (1..3).map { |idx| File.join('/tmp', "contained_#{idx}.#{EXTENSIONS[idx-1]}") }
      contained_paths.each { |path| File.open(path, 'w') { |f| f.puts "Hello #{path}" } }
      paths_to_add = contained_paths.map { |path| File.basename(path) }.join(' ')
      system "cd '#{File.dirname(@file_to_test)}'; zip '#{File.basename(@file_to_test)}' '#{paths_to_add}' > /dev/null 2>&1"
      contained_paths.each { |path| FileUtils.rm path }

      za = MdsFileUtils::ZipAnalyzer.new(@file_to_test)
      
      za.is_composite?.should be_falsey
    end

    it "should respond with true when the file listing has no directory entry and all files are in a subdirectory" do
      za = MdsFileUtils::ZipAnalyzer.new('/tmp/baloney.zip')
      za.should_receive(:valid_zip?).and_return(true)

      mock_unzipper = double("UnzipmeUnzipper")
      mock_unzipper.should_receive(:file_list).
        and_return([ 'Abaqis Upload/Transmitted Files-01.zip',
                     'Abaqis Upload/Transmitted Files-02.zip',
                     'Abaqis Upload/Transmitted Files-03.zip' ])

      UnzipmeUnzipper.should_receive(:new).with('/tmp/baloney.zip').and_return(mock_unzipper)

      expect(za.is_composite?).to be_truthy
    end

    it "should respond with true when the file listing has a directory entry and all files are in a subdirectory" do
      za = MdsFileUtils::ZipAnalyzer.new('/tmp/baloney.zip')
      za.should_receive(:valid_zip?).and_return(true)

      mock_unzipper = double("UnzipmeUnzipper")
      mock_unzipper.should_receive(:file_list).
        and_return([ 'Abaqis Upload/',
                     'Abaqis Upload/Transmitted Files-01.zip',
                     'Abaqis Upload/Transmitted Files-02.zip',
                     'Abaqis Upload/Transmitted Files-03.zip' ])

      UnzipmeUnzipper.should_receive(:new).with('/tmp/baloney.zip').and_return(mock_unzipper)

      expect(za.is_composite?).to be_truthy
    end

    it "should respond with true when the file listing has at least one .zip file" do
      za = MdsFileUtils::ZipAnalyzer.new('/tmp/baloney.zip')
      za.should_receive(:valid_zip?).and_return(true)

      mock_unzipper = double("UnzipmeUnzipper")
      mock_unzipper.should_receive(:file_list).
        and_return([ 'Files0-00.zip',
                     'Abaqis Upload/Transmitted Files-01.xml',
                     'Abaqis Upload/Transmitted Files-02.png',
                     'Abaqis Upload/Transmitted Files-03' ])

      UnzipmeUnzipper.should_receive(:new).with('/tmp/baloney.zip').and_return(mock_unzipper)

      expect(za.is_composite?).to be_truthy
    end

    it "should respond with false when the file listing doesn't have at least one .zip file" do
      za = MdsFileUtils::ZipAnalyzer.new('/tmp/baloney.zip')
      za.should_receive(:valid_zip?).and_return(true)

      mock_unzipper = double("UnzipmeUnzipper")
      mock_unzipper.should_receive(:file_list).
        and_return([ 'Files0-00.iso',
                     'Abaqis Upload/Transmitted Files-01.xml',
                     'Abaqis Upload/Transmitted Files-02.png',
                     'Abaqis Upload/Transmitted Files-03' ])

      UnzipmeUnzipper.should_receive(:new).with('/tmp/baloney.zip').and_return(mock_unzipper)

      expect(za.is_composite?).to be_falsey
    end

  end

  context '#non_mds_data_in_composite?' do

    before(:each) do
      @cut = MdsFileUtils::ZipAnalyzer.new('afile')
    end

    it "should return true if the list of files in the zip file contain entries that are not ZIP files" do
      @cut.should_receive(:file_list).and_return([ 'thing1.zip', 'thing2.jpg', 'dir/thing3.zip' ])
      expect(@cut.non_mds_data_in_composite?).to be_truthy
    end

    it "should return false if the list of files in the zip file contain only directories and ZIP files" do
      @cut.should_receive(:file_list).and_return([ 'thing1.zip', 'dir/', 'dir2\\', 'dir/thing2.ZIP' ])
      expect(@cut.non_mds_data_in_composite?).to be_falsey
    end

  end

end

